-- TODO Remove expiration information from tooltip

function firstCharIsNarrow(text)
	if (not text) then
		return false;
	end
	local firstChar = string.sub(text, 1, 1);
	return firstChar == "1"
		or levelTextFirstChar == "i"
		or levelTextFirstChar == "I"
		or levelTextFirstChar == "l"
		or levelTextFirstChar == ""
		or levelTextFirstChar == "!";
end

function OrderHallMissionFrame_MissionTab_MissionList_Update_PostHook(self)
	local missions;
	if (self.showInProgress) then
		missions = self.inProgressMissions;
	else
		missions = self.availableMissions;
	end
	local numMissions = #missions;
	local scrollFrame = self.listScroll;
	local offset = HybridScrollFrame_GetOffset(scrollFrame);
	local buttons = scrollFrame.buttons;
	local numButtons = #buttons;

	if (SFE_ORDERHALL_MISSIONS_ENHANCED) then
		for i = 1, numButtons do
			local button = buttons[i];
			local index = offset + i;
			if (index <= numMissions) then
				local mission = missions[index];

				-- Time details
				if (not mission.inProgress) then
					local duration = mission.duration;
					if (mission.durationSeconds >= GARRISON_LONG_MISSION_TIME) then
						duration = string.format(GARRISON_LONG_MISSION_TIME_FORMAT, duration);
					end
					local durationText = string.format(PARENS_TEMPLATE, duration);
					local durationTextLength = string.len(durationText);
					if (string.sub(durationText, durationTextLength) == ")") then
						local expirationMessage = "";
						if (mission.offerEndTime) then
							local remainingSeconds = mission.offerEndTime - GetTime();
							local colorCode, colorCodeEnd = "", "";
							if (remainingSeconds < 8 * 3600) then
								colorCode, colorCodeEnd = RED_FONT_COLOR_CODE, FONT_COLOR_CODE_CLOSE;
							elseif (remainingSeconds < 24 * 3600) then
								colorCode, colorCodeEnd = YELLOW_FONT_COLOR_CODE, FONT_COLOR_CODE_CLOSE;
							end
							expirationMessage = ", "..colorCode..string.format(SFE_EXPIRES_IN, mission.offerTimeRemaining)..colorCodeEnd;
						end
						button.Summary:SetText(string.sub(durationText, 2, durationTextLength - 1)..expirationMessage);
					end
				end
				button.Title:SetPoint("LEFT", 165, 10);
				button.Title:SetWidth(655 - #mission.rewards * 65);
				button.Summary:ClearAllPoints();
				button.Summary:SetPoint("TOPLEFT", button.Title, "BOTTOMLEFT", 0, -4);

				-- Level
				local levelTextY = 0;
				if (mission.isMaxLevel and mission.iLevel > 0) then
					button.Level:SetText(mission.iLevel);
					button.ItemLevel:SetText(ITEM_LEVEL_ABBR);
					levelTextY = - button.ItemLevel:GetHeight() / 2;
				end
				button.Level:SetPoint("CENTER", button, "LEFT", 40, levelTextY);
				-- WoW calculates fonts as monospaced, which visually is false, since '1' is thin it doesn't look centered
				local iLevelTextX = 0;
				if (firstCharIsNarrow(button.Level:GetText())) then
					iLevelTextX = iLevelTextX + 3;
				end
				if (firstCharIsNarrow(button.ItemLevel:GetText())) then
					iLevelTextX = iLevelTextX - 2;
				end
				button.ItemLevel:ClearAllPoints();
				button.ItemLevel:SetPoint("BOTTOM", button.Level, "TOP", iLevelTextX, 2);

				-- Rarity
				if (mission.isRare) then
					button.Level:SetTextColor(0.098, 0.537, 0.969, 1.0);
					button.ItemLevel:SetTextColor(0.098, 0.537, 0.969, 1.0);
				else
					button.Level:SetTextColor(0.84, 0.72, 0.57, 1.0);
					button.ItemLevel:SetTextColor(0.84, 0.72, 0.57, 1.0);
				end
				button.RareText:Hide();

				-- Rewards
				local missionRewards = {};
				local missionRewardCount = 0;
				if (mission.rewards) then
					for i = 1, #mission.rewards do
						missionRewardCount = missionRewardCount + 1;
						missionRewards[missionRewardCount] = mission.rewards[i];
					end
				end
				if (mission.overmaxRewards) then
					for i = 1, #mission.overmaxRewards do
						missionRewardCount = missionRewardCount + 1;
						missionRewards[missionRewardCount] = mission.overmaxRewards[i];
					end
				end
				if (button.Title:GetWidth() > 655 - 12 - missionRewardCount * 56) then
					button.Title:SetWidth(655 - 12 - missionRewardCount * 56);
				end
				if (button.Summary:GetWidth() > 655 - 12 - missionRewardCount * 56) then
					button.Summary:SetWidth(655 - 12 - missionRewardCount * 56);
				end
				GarrisonMissionButton_SetRewards(button, missionRewards);
				for i = #button.Rewards, 1, -1 do
					button.Rewards[i]:SetSize(48, 48);
					if (not button.Rewards[i].originalRightPoint) then
						for j = 1, button.Rewards[i]:GetNumPoints() do
							local point, relativeTo, relativePoint, xOffset, yOffset = button.Rewards[i]:GetPoint(j);
							if (point == "RIGHT") then
								button.Rewards[i].originalRightPoint = { relativeTo, relativePoint, xOffset, yOffset };
							end
						end
					end
					if (i < #button.Rewards) then
						button.Rewards[i]:SetPoint("RIGHT", button.Rewards[i + 1], "LEFT", -8, 0);
					else
						button.Rewards[i]:SetPoint("RIGHT", button, "RIGHT", -12, 0);
					end
					button.Rewards[i].Icon:SetMask("Interface\\ICONS\\INV_Helmet_47");
					if (not button.Rewards[i].bonusIcon) then
						button.Rewards[i].bonusIcon = CreateFrame("Frame", nil, button);
						button.Rewards[i].bonusIcon:SetSize(28, 28);
						button.Rewards[i].bonusIcon:SetPoint("CENTER", button.Rewards[i], "TOPRIGHT", -8, -8);
						button.Rewards[i].bonusIcon:SetHitRectInsets(4, 4, 1, 4);
						button.Rewards[i].bonusIcon.texture = button.Rewards[i].bonusIcon:CreateTexture(nil, "ARTWORK", nil, 2);
						button.Rewards[i].bonusIcon.texture:SetAtlas("honorsystem-icon-bonus");
						button.Rewards[i].bonusIcon.texture:SetSize(28, 28);
						button.Rewards[i].bonusIcon.texture:SetPoint("CENTER", button.Rewards[i].bonusIcon, "CENTER", 0, 0);
						button.Rewards[i].bonusIcon:SetScript("OnEnter", function(self, motion)
								GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
								GameTooltip:SetText(BONUS_REWARDS, nil, nil, nil, nil, true);
							end);
						button.Rewards[i].bonusIcon:SetScript("OnLeave", GameTooltip_Hide);
					end
					if (i > #mission.rewards) then
						button.Rewards[i].bonusIcon:Show();
					else
						button.Rewards[i].bonusIcon:Hide();
					end
				end
			end
		end
	else
		for i = 1, numButtons do
			local button = buttons[i];
			local index = offset + i;
			if (index <= numMissions) then
				local mission = missions[index];

				-- Level
				if (mission.isRare and mission.isMaxLevel and mission.iLevel > 0) then
					button.Level:SetPoint("CENTER", button, "TOPLEFT", 40, -22);
				else
					button.Level:SetPoint("CENTER", button, "TOPLEFT", 40, -36);
				end
				button.ItemLevel:ClearAllPoints();
				button.ItemLevel:SetPoint("TOP", button.Level, "BOTTOM", 0, 2);

				-- Rarity
				button.Level:SetTextColor(0.84, 0.72, 0.57, 1.0);
				button.ItemLevel:SetTextColor(0.84, 0.72, 0.57, 1.0);

				-- Rewards
				for i = 1, #button.Rewards do
					button.Rewards[i]:SetSize(72, 72);
					if (button.Rewards[i].originalRightPoint) then
						button.Rewards[i]:SetPoint("RIGHT", button.Rewards[i].originalRightPoint[1], button.Rewards[i].originalRightPoint[2], button.Rewards[i].originalRightPoint[3], button.Rewards[i].originalRightPoint[4]);
					end
					button.Rewards[i].Icon:SetMask(nil);
					if (button.Rewards[i].bonusIcon) then
						button.Rewards[i].bonusIcon:Hide();
					end
				end
			end
		end
	end
end

--function GarrisonMissionButton_OnEnter_Enhanced(self, button, oldFunction)
--	oldFunction(self, button);
--
--	if (self.info == nil) then
--		oldFunction(self, button);
--		return;
--	end
--end

local SfeOrderHallController = CreateFrame("Frame");
SfeOrderHallController:RegisterEvent("ADDON_LOADED");
SfeOrderHallController:SetScript("OnEvent", function(self, event, name)
		if ((event == "ADDON_LOADED") and (name == "Blizzard_OrderHallUI")) then
			if (SFE_ORDERHALL_MISSIONS_ENHANCED == nil) then
				SFE_ORDERHALL_MISSIONS_ENHANCED = true;
			end

			if (OrderHallMissionsEnhancedCheckbox == nil) then
				OrderHallMissionsEnhancedCheckbox = CreateFrame("CheckButton", "OrderHallMissionsEnhancedCheckbox", OrderHallMissionFrame, "OptionsBaseCheckButtonTemplate");
				OrderHallMissionFrame.EnhancedCheckbox = OrderHallMissionsEnhancedCheckbox;
				OrderHallMissionsEnhancedCheckbox:SetHitRectInsets(0, 0, 0, 0);

				OrderHallMissionsEnhancedCheckbox:SetPoint("RIGHT", OrderHallMissionFrame.CloseButton, "LEFT", 7, 0);
				OrderHallMissionsEnhancedCheckbox:SetAlpha(0.1);
				OrderHallMissionsEnhancedCheckbox:SetScript("OnClick", function(self, button, down)
						SFE_ORDERHALL_MISSIONS_ENHANCED = self:GetChecked();
						OrderHallMissionFrame.MissionTab.MissionList:Update(OrderHallMissionFrame.MissionTab.MissionList);
						if (SFE_ORDERHALL_MISSIONS_ENHANCED) then
							PlaySound("igMainMenuOptionCheckBoxOn");
						else
							PlaySound("igMainMenuOptionCheckBoxOff");
						end
					end);
				OrderHallMissionsEnhancedCheckbox:SetScript("OnEnter", function(self, motion)
						self:SetAlpha(1.0);
						GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
						GameTooltip:SetText(SFE_ENHANCED_UI, nil, nil, nil, nil, true);
					end);
				OrderHallMissionsEnhancedCheckbox:SetScript("OnLeave", function(self, motion)
						self:SetAlpha(0.1);
						GameTooltip_Hide(self, motion);
					end);
			end
			OrderHallMissionsEnhancedCheckbox:SetChecked(SFE_ORDERHALL_MISSIONS_ENHANCED);

			local oldOrderHallMissionFrame_MissionTab_MissionList_Update = OrderHallMissionFrame.MissionTab.MissionList.Update;
			function OrderHallMissionFrame.MissionTab.MissionList:Update()
				oldOrderHallMissionFrame_MissionTab_MissionList_Update(self);
				OrderHallMissionFrame_MissionTab_MissionList_Update_PostHook(self);
			end
--			local oldGarrisonMissionButton_OnEnter = GarrisonMissionButton_OnEnter;
--			function GarrisonMissionButton_OnEnter(self, button)
--				GarrisonMissionButton_OnEnter_Enhanced(self, button, oldGarrisonMissionButton_OnEnter);
--			end
		end
	end);
